import datetime
from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import Unicode, UnicodeText
from sqlalchemy import ForeignKey, Boolean
from sqlalchemy import Date, Time
from sqlalchemy import Enum
from sqlalchemy import PickleType
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship

from ..alchemy import TimeStampMixin

# ########################## RULES ##############################
# A User has a single Contact (one contact:many users).
#
#   An Event depends on an EventType.  An EventType must be
# created before an Event is created.
#
# A "host" must have a Venue to promote an Event.
#
# //Multiple "hosts" can promote Event(s) at the same Venue
#
# //An Event can have multiple Venues (such as a festival)
#
# //When a "host" claims a Venue, this host selects the other
# //hosts that can promote Event(s) at this Venue.
#
# The three commented options above are too time consuming
# to implement at this point, while also needing some
# coordination and design considerations outside the scope
# of the website.
#
# I have just decided to layout the basic data model for
# festivals.  A festival is a collection of events that
# will span for a length of time (a day, a weekend, a week
# or two).  This will allow user to host a festival and
# invite other users to host events.
#
#
################################################################


RoleType = Enum('admin', 'host', 'user', 'guest', name='roletype')


class AddressMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'addresses'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def street(self):
        return Column(Unicode(150))

    @declared_attr
    def street2(self):
        return Column(Unicode(150))

    @declared_attr
    def city(self):
        return Column(Unicode(50))

    @declared_attr
    def state(self):
        return Column(Unicode(2))

    @declared_attr
    def zip(self):
        return Column(Unicode(10))


class ContactMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'addresses'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def firstname(self):
        return Column(Unicode(50))

    @declared_attr
    def lastname(self):
        return Column(Unicode(50))

    @declared_attr
    def email(self):
        return Column(Unicode(150))

    @declared_attr
    def phone(self):
        return Column(Unicode(20))

    @declared_attr
    def users(self):
        return relationship('User')


class AudienceMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'audiences'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def location(self):
        return Column(Unicode(50))

    def __init__(self, location=None):
        self.location = location


class VenueMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'venues'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def name(self):
        return Column(Unicode(50))

    @declared_attr
    def audience_id(self):
        return Column(Integer, ForeignKey('audiences.id'))

    @declared_attr
    def address_id(self):
        return Column(Integer, ForeignKey('addresses.id'))

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey('users.id'))

    @declared_attr
    def description(self):
        return Column(UnicodeText)

    def __init__(self, user_id, name=None, audience_id=None):
        self.user_id = user_id
        self.name = name
        self.audience_id = audience_id


class EventTypeMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'event_types'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def name(self):
        return Column(Unicode(50), unique=True)

    def __init__(self, name=None):
        self.name = name


# FIXME
class EventTypeColorMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'event_type_colors'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def color(self):
        return Column(Unicode(50))

    def __init__(self, color):
        self.color = color


class EventMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'events'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def start_date(self):
        return Column(Date)

    @declared_attr
    def start_time(self):
        return Column(Time)

    # FIXME should I use hybrid attribute?
    @declared_attr
    def starts_at(self):
        return datetime.datetime.combine(self.start_date, self.start_time)

    @declared_attr
    def end_date(self):
        return Column(Date)

    @declared_attr
    def end_time(self):
        return Column(Time)

    # FIXME should I use hybrid attribute?
    @declared_attr
    def ends_at(self):
        return datetime.datetime.combine(self.end_date, self.end_time)

    @declared_attr
    def all_day(self):
        return Column(Boolean('all_day_event'), default=False)

    @declared_attr
    def title(self):
        return Column(Unicode(255))

    @declared_attr
    def description(self):
        return Column(UnicodeText)

    @declared_attr
    def originator(self):
        return Column(Integer, ForeignKey('users.id'))

    @declared_attr
    def event_type(self):
        return Column(Integer, ForeignKey('event_types.id'))

    @declared_attr
    def venue_id(self):
        return Column(Integer, ForeignKey('venues.id'))

    @declared_attr
    def venue(self):
        return relationship('Venue', secondary='event_venues',
                            backref='events')

    def __init__(self, originator):
        self.originator = originator


class EventVenueMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'event_venues'

    @declared_attr
    def event_id(self):
        return Column(Integer, ForeignKey('events.id'), primary_key=True)

    @declared_attr
    def venue_id(self):
        return Column(Integer, ForeignKey('venues.id'), primary_key=True)

    def __init__(self, event_id, venue_id):
        self.event_id = event_id
        self.venue_id = venue_id


class VenueInfoMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'venue_info'

    @declared_attr
    def id(self):
        return Column(Integer, ForeignKey('venues.id'), primary_key=True)

    @declared_attr
    def info(self):
        return Column(PickleType)

    def __init__(self, id, info):
        self.id = id
        self.info = info


class FestivalMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'festivals'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True)


class FestivalEventMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'festival_events'

    @declared_attr
    def festival_id(self):
        return Column(Integer, ForeignKey('festival.id'), primary_key=True)

    @declared_attr
    def event_id(self):
        return Column(Integer, ForeignKey('events.id'), primary_key=True)


class UserInfoMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'commix_user_info'

    @declared_attr
    def id(self):
        return Column(Integer, ForeignKey('users.id'), primary_key=True)

    @declared_attr
    def contact_id(self):
        return Column(Integer, ForeignKey('contacts.id'))

    @declared_attr
    def role(self):
        return Column(RoleType)

    @declared_attr
    def default_audience_id(self):
        return Column(Integer, ForeignKey('audiences.id'))

    @declared_attr
    def contact(self):
        return relationship('Contact')



import os
import json
import subprocess

from unipath.path import Path as path


def assert_git_directory(directory):
    directory = path(directory)
    assert directory.isdir()
    cmd = ['git', '-C', directory, 'rev-parse']
    subprocess.check_call(cmd)


def clone_repo(uri, dest, branch=None, quiet=True, bare=False,
               mirror=False, verbose=False):
    cmd = ['git', 'clone']
    if quiet:
        cmd.append('--quiet')
    if branch is not None:
        cmd += ['--branch', branch]
    if bare:
        cmd.append('--bare')
    if mirror:
        cmd.append('--mirror')
    cmd += [uri, dest]
    if verbose:
        print("Clone command: %s" % ' '.join(cmd))
    subprocess.check_call(cmd)


def fetch_all(directory, quiet=False):
    assert_git_directory(directory)
    prefix = ['git', '-C', directory]
    cmd = prefix + ['fetch', '--all']
    subprocess.check_call(cmd)


def update_repo(directory, quiet=False, all=True):
    assert_git_directory(directory)
    assert os.path.isdir(directory)
    prefix = ['git', '-C', directory]
    current_sha_cmd = prefix + ['rev-parse', 'HEAD']
    current_sha = subprocess.check_output(current_sha_cmd).strip()
    cmd = prefix + ['fetch']
    if quiet:
        cmd.append('--quiet')
    if all:
        cmd.append('--all')
    upstream_sha_cmd = prefix + ['rev-parse', 'FETCH_HEAD']
    upstream_sha = subprocess.check_output(upstream_sha_cmd).strip()
    if current_sha != upstream_sha:
        subprocess.check_call(prefix + ['merge', upstream_sha])


def update_mirrored_repo(directory):
    cmd = ['git', '-C', directory, 'remote', 'update']
    subprocess.check_call(cmd)


# FIXME - this is git-annex
def check_remote_present(directory, name):
    assert_git_directory(directory)
    oldpwd = os.getcwd()
    os.chdir(directory)
    cmd = ['git-annex', 'info', '--fast', '--json', name]
    out = subprocess.check_output(cmd)
    os.chdir(oldpwd)
    return json.loads(out)



import os
import subprocess

from unipath.path import Path as path
from git import Repo

from .githubdb import GitHubUser, GitHubRepo


class RepoManager(object):
    def __init__(self, session, user_id):
        self.session = session
        self.ghclient = None
        self.repo_path = None
        self.user = session.query(GitHubUser).get(user_id)

    def set_repo_path(self, repo_path):
        repo_path = path(repo_path)
        if not repo_path.isdir():
            raise RuntimeError("%s doesn't exist." % repo_path)
        self.repo_path = repo_path

    def set_github_client(self, ghclient):
        self.ghclient = ghclient

    def get_my_repos(self):
        user_id = self.user.id
        q = self.session.query(GitHubRepo).filter_by(owner_id=user_id)
        return q.all()

    def local_reponame(self, dbrepo):
        reponame = dbrepo.full_name
        if not reponame.startswith(self.user.login):
            reponame = os.path.join('others', dbrepo.name)
        reponame = os.path.join(self.repo_path, '%s.git' % reponame)
        return reponame

    def local_repo_exists(self, dbrepo):
        return os.path.isdir(self.local_reponame(dbrepo))

    def destroy_repo(self, dbrepo):
        if self.local_repo_exists(dbrepo):
            dirname = self.local_reponame(dbrepo)
            cmd = ['rm', '-fr', str(dirname)]
            subprocess.check_call(cmd)

    def clone_repo(self, dbrepo, reponame=None, size_limit=None):
        if size_limit is not None:
            if dbrepo.size > size_limit:
                print("%s too big." % dbrepo.full_name, dbrepo.size)
                return
        clone_url = dbrepo.pickle.clone_url
        if reponame is None:
            reponame = self.local_reponame(dbrepo)
        repo = Repo.clone_from(clone_url, reponame, bare=True)
        return repo

    def local_repo(self, dbrepo):
        return Repo(self.local_reponame(dbrepo))

    def _clone_repolist(self, repolist, size_limit=None):
        for repo in repolist:
            reponame = self.local_reponame(repo)
            if not os.path.isdir(reponame):
                self.clone_repo(repo,
                                reponame=reponame,
                                size_limit=size_limit)

    def clone_my_repos(self, size_limit=1000):
        my_repos = self.get_my_repos()
        self._clone_repolist(my_repos, size_limit=size_limit)

    def clone_other_repos(self, size_limit=1000):
        query = self.session.query(GitHubRepo)
        others = query.filter(GitHubRepo.owner_id != self.user.id)
        self._clone_repolist(others, size_limit=size_limit)

    def update_from_github(self):
        pass



from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import Unicode
from sqlalchemy import ForeignKey, PickleType, DateTime
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship

from hornstone.alchemy import TimeStampMixin


class FeedMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'rssfeeds'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def name(self):
        return Column(Unicode(100), unique=True)

    @declared_attr
    def url(self):
        return Column(Unicode(100), unique=True)


class FeedDataMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'rssdata'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def feed_id(self):
        return Column(Integer, ForeignKey('rssfeeds.id'))

    @declared_attr
    def content(self):
        return Column(PickleType)

    @declared_attr
    def retrieved(self):
        return Column(DateTime)

    @declared_attr
    def feed(self):
        return relationship('Feed')



import os
from configparser import ConfigParser
from io import StringIO


DEFAULT_CONFIG_TEXT = """\
# Configuration file for hornstone

[main]
# put something here....
"""

default_file = os.path.expanduser('~/.config/hornstone/hornstone.ini')
config = ConfigParser()
if not os.path.isfile(default_file):
    dirname = os.path.dirname(default_file)
    if not os.path.isdir(dirname):
        os.makedirs(dirname)
    print('Generating default config...')
    config.readfp(StringIO(DEFAULT_CONFIG_TEXT))
    with open(default_file, 'w') as outfile:
        config.write(outfile)

config.read([default_file])



import unittest


def init_user_database(engine):
    from .alchemy import Base
    from .models.usergroup import UserMixin, GroupMixin, UserGroupMixin

    class User(Base, UserMixin):
        pass

    class Group(Base, GroupMixin):
        pass

    class GroupUser(Base, UserGroupMixin):
        pass
    models = dict(User=User,
                  Group=Group, GroupUser=GroupUser)
    Base.metadata.create_all(engine)
    return models


class ChunksTest(unittest.TestCase):
    def test_chunks1(self):
        from .base import chunks
        result = list(chunks(b'abcdefghi', 4))
        expected = [b'abcd', b'efgh', b'i']
        self.assertEqual(result, expected)

    def test_chunks2(self):
        from .base import chunks
        result = list(chunks(b'abcdefghi', 3))
        expected = [b'abc', b'def', b'ghi']
        self.assertEqual(result, expected)


class BaseAlchemyTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        from sqlalchemy import engine_from_config
        from sqlalchemy.orm import sessionmaker
        settings = {'sqlalchemy.url': 'sqlite:///:memory:'}
        cls.engine = engine_from_config(settings, 'sqlalchemy.')
        factory = sessionmaker()
        factory.configure(bind=cls.engine)
        cls.session = factory()

    @classmethod
    def tearDownClass(cls):
        from .alchemy import Base
        Base.metadata.drop_all(cls.engine)


class BaseUserGroupTest(BaseAlchemyTest):
    @classmethod
    def setUpClass(cls):
        super(BaseUserGroupTest, cls).setUpClass()
        cls.models = init_user_database(cls.engine)

    @classmethod
    def tearDownClass(cls):
        from .alchemy import Base
        Base.metadata.drop_all(cls.engine)


class SimpleUserTest(BaseUserGroupTest):
    @classmethod
    def setUpClass(cls):
        super(SimpleUserTest, cls).setUpClass()
        user = cls.models['User']()
        user.username = 'admin'
        cls.session.add(user)
        cls.session.commit()

    def test_user_created(self):
        User = self.models['User']
        user = self.session.query(User).filter_by(username='admin').first()
        self.assertEqual(user.username, 'admin')

    def test_user_id(self):
        from uuid import UUID
        User = self.models['User']
        user = self.session.query(User).filter_by(username='admin').first()
        result = type(user.id)
        expected = UUID
        self.assertEqual(result, expected)



import os
import hashlib


# http://code.activestate.com/recipes/576620-changedirectory-context-manager/
class WorkingDirectory(object):
    def __init__(self, directory):
        self._dir = directory
        self._cwd = os.getcwd()
        self._pwd = self._cwd

    @property
    def current(self):
        return self._cwd

    @property
    def previous(self):
        return self._pwd

    @property
    def relative(self):
        c = self._cwd.split(os.path.sep)
        p = self._pwd.split(os.path.sep)
        shortlen = min(len(c), len(p))
        i = 0
        while i < shortlen and c[i] == p[i]:
            i += 1
        return os.path.normpath(os.path.join(*(['.'] + (['..'] * (len(c) - i)) + p[i:])))  # noqa

    def __enter__(self):
        self._pwd = self._cwd
        os.chdir(self._dir)
        self._cwd = os.getcwd()
        return self

    def __exit__(self, *args):
        os.chdir(self._pwd)
        self._cwd = self._pwd


def chunks(l, n):
    """ Yield successive n-sized chunks from l.
    """
    for i in range(0, len(l), n):
        yield l[i:i + n]


def get_sha256sum(fileobj):
    s = hashlib.new('sha256')
    block = fileobj.read(4096)
    while block:
        s.update(block)
        block = fileobj.read(4096)
    return s.hexdigest()


def get_sha256sum_string(string):
    s = hashlib.new('sha256')
    s.update(string)
    return s.hexdigest()


def trailing_slash(dirname):
    if not dirname.endswith('/'):
        return '%s/' % dirname
    return dirname


def remove_trailing_slash(pathname):
    while pathname.endswith('/'):
        pathname = pathname[:-1]
    return pathname


def parse_config_lines(filename):
    return [line.strip() for line in open(filename)
            if line.strip() and not line.strip().startswith('#')]



from sqlalchemy import Column, ForeignKey

# column types
from sqlalchemy import (
    Integer,
    Unicode,
    UnicodeText,
    Enum,
    )
from sqlalchemy.orm import relationship

from hornstone.alchemy import Base, SerialBase

####################################
# Data Types                     ###
####################################

HubbyFileType = Enum('agenda', 'minutes', 'attachment',
                     name='hubbyfile_type_enum')

####################################
# Tables                         ###
####################################


class ScummVMCompany(Base, SerialBase):
    __tablename__ = 'scummvm_companies'
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(200), unique=True)


class ScummVMGame(Base, SerialBase):
    __tablename__ = 'scummvm_games'
    id = Column(Unicode(50), primary_key=True)
    name = Column(Unicode(300))
    company_id = Column(Integer, ForeignKey('scummvm_companies.id'))
    support_level = Column(Unicode(100))
    notes = Column(UnicodeText)


####################################
# Relationships                  ###
####################################

ScummVMGame.company = relationship(ScummVMCompany)



import uuid
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy_utils import UUIDType

from ..alchemy import TimeStampMixin


class BaseUUIDMixin(TimeStampMixin):
    @declared_attr
    def id(self):
        return sa.Column(UUIDType, primary_key=True,
                         default=uuid.uuid4())


class BaseIdMixin(TimeStampMixin):
    @declared_attr
    def id(self):
        return sa.Column(sa.Integer, primary_key=True, autoincrement=True)


class UserOwnedMixin(TimeStampMixin):
    @declared_attr
    def user_id(self):
        return sa.Column(UUIDType, sa.ForeignKey('users.id'))

# FIXME these name mixins are probably not useful


class BaseShortNameIdMixin(BaseIdMixin):
    @declared_attr
    def name(self):
        return sa.Column(sa.Unicode(50), unique=True)


class BaseShortNameUUIDMixin(BaseUUIDMixin):
    @declared_attr
    def name(self):
        return sa.Column(sa.Unicode(50), unique=True)


class BaseLongNameIdMixin(BaseIdMixin):
    @declared_attr
    def name(self):
        return sa.Column(sa.Unicode(255), unique=True)


class BaseNameIdMixin(BaseIdMixin):
    @declared_attr
    def name(self):
        return sa.Column(sa.Unicode, unique=True)



import os
import uuid as UUID

from unipath.path import Path as path

from hornstone.base import get_sha256sum_string

FILE_EXTENSIONS = ['jpg', 'png', 'gif']


class ImageRepo(object):
    def __init__(self, repo_path):
        self.repo_path = path(repo_path)

    def _get_top_bottom(self, uuid):
        h = uuid.hex
        return tuple(h.split(h[2:-2]))

    def _repo_dir(self, checksum):
        top, bottom = self._get_top_bottom(checksum)
        return os.path.join(self.repo_path, top, bottom)

    def relname(self, checksum, ext):
        # top, bottom = self._get_top_bottom(checksum)
        # return os.path.join(top, bottom, checksum)
        return '%s.%s' % (checksum, ext)

    def filename(self, checksum, ext):
        # dirname = self._repo_dir(checksum)
        # return os.path.join(dirname, checksum)
        return os.path.join(self.repo_path, '%s.%s' % (checksum, ext))

    def file_exists(self, checksum, ext):
        filename = self.filename(checksum, ext)
        return os.path.isfile(filename)

    def get_checksum_content(self, content):
        return get_sha256sum_string(content)

    def import_content(self, content, ext):
        checksum = self.get_checksum_content(content)
        filename = self.filename(checksum, ext)
        if os.path.isfile(filename):
            raise RuntimeError("File already exists %s" % checksum)
        with open(filename, 'wb') as outfile:
            outfile.write(content)

    def open(self, checksum, ext, mode='rb'):
        filename = self.filename(checksum, ext)
        return open(filename, mode)

    def delete(self, checksum, ext):
        if self.file_exists(checksum, ext):
            os.remove(self.filename(checksum, ext))

    def delete_all(self):
        for basename in os.listdir(self.repo_path):
            filename = os.path.join(self.repo_path, basename)
            if os.path.isfile(filename):
                os.remove(filename)


class UrlRepo(object):
    def __init__(self, repo_path):
        self.repo_path = path(repo_path)

    def get_uuid(self, url):
        return UUID.uuid5(UUID.NAMESPACE_URL, bytes(url))

    def _get_top_bottom(self, uuid):
        h = uuid.hex
        return tuple(h.split(h[2:-2]))

    def _repos_dir(self, url, uuid=None):
        if uuid is None:
            uuid = self.get_uuid(url)
        top, bottom = self._get_top_bottom(uuid)
        return os.path.join(self.repo_path, top, bottom)

    def _basename(self, url, uuid):
        for e in FILE_EXTENSIONS:
            if url.endswith('.%s' % e):
                return '%s.%s' % (uuid.hex, e)
        return uuid.hex

    def _repos_name(self, url):
        uuid = self.get_uuid(url)
        dirname = self._repos_dir(url, uuid)
        basename = self._basename(url, uuid)
        return os.path.join(dirname, basename)

    def relname(self, url):
        uuid = self.get_uuid(url)
        top, bottom = self._get_top_bottom(uuid)
        return os.path.join(top, bottom, self._basename(url, uuid))

    def file_exists(self, url):
        return os.path.isfile(self._repos_name(url))

    def filename(self, url):
        return self._repos_name(url)

    def open_file(self, url, mode='rb'):
        filename = self._repos_name(url)
        dirname = os.path.dirname(filename)
        if not os.path.isdir(dirname):
            os.makedirs(dirname)
        return open(filename, mode)



from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import Unicode, UnicodeText
from sqlalchemy import PickleType
from sqlalchemy import DateTime
from sqlalchemy.ext.declarative import declared_attr

from hornstone.alchemy import TimeStampMixin


class CeleryTaskMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'celery_taskmeta'

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True, nullable=False)

    @declared_attr
    def task_id(self):
        return Column(Unicode(255))

    @declared_attr
    def status(self):
        return Column(Unicode(50))

    @declared_attr
    def result(self):
        return Column(PickleType)

    @declared_attr
    def date_done(self):
        return Column(DateTime)

    @declared_attr
    def traceback(self):
        return Column(UnicodeText)



from datetime import datetime, date


# https://stackoverflow.com/a/22238613
def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    if isinstance(obj, bytes):
        return obj.decode()
    raise TypeError("Type %s not serializable" % type(obj))



import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr

from .base import BaseLongNameIdMixin
from .base import UserOwnedMixin

DocType = sa.Enum('markdown', 'html',
                  name='document_type_enum')


class BaseDocumentMixin(BaseLongNameIdMixin):
    @declared_attr
    def title(self):
        return sa.Column(sa.Unicode(500))

    @declared_attr
    def description(self):
        return sa.Column(sa.Unicode(500))

    @declared_attr
    def doctype(self):
        return sa.Column(DocType, default='markdown')

    @declared_attr
    def content(self):
        return sa.Column(sa.UnicodeText)


class SiteDocumentMixin(BaseDocumentMixin):
    @declared_attr
    def __tablename__(self):
        return 'site_documents'

    @declared_attr
    def category(self):
        return sa.Column(sa.Unicode(50), default='site')


class UserDocumentMixin(BaseDocumentMixin, UserOwnedMixin):
    @declared_attr
    def __tablename__(self):
        return 'user_documents'



from configparser import ConfigParser
from io import StringIO

from sqlalchemy import Column
from sqlalchemy import Integer, Boolean
from sqlalchemy import Unicode, UnicodeText
from sqlalchemy import ForeignKey, PickleType
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship
from sqlalchemy_utils import UUIDType

from ..alchemy import TimeStampMixin
from .base import BaseIdMixin, BaseShortNameIdMixin
from .base import BaseUUIDMixin
from .base import BaseShortNameUUIDMixin


class UserMixin(BaseUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'users'

    @declared_attr
    def username(self):
        return Column(Unicode(50), unique=True)

    @declared_attr
    def fullname(self):
        return Column(Unicode(150), unique=True)

    @declared_attr
    def email(self):
        return Column(Unicode(150), unique=True)

    @declared_attr
    def active(self):
        return Column(Boolean(name='user_active'), default=True)

    @declared_attr
    def password(self):
        return Column(Unicode(150))

    @declared_attr
    def settings(self):
        return Column(PickleType)

    @property
    def user_name(self):
        return super(TimeStampMixin, self).username

    def get_groups(self):
        return [g.name for g in self.groups]


class UserConfigMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'user_config'

    @declared_attr
    def id(self):
        return Column(UUIDType, ForeignKey('users.id'), primary_key=True)

    @declared_attr
    def text(self):
        return Column(UnicodeText)

    def get_config(self):
        c = ConfigParser()
        c.readfp(StringIO(self.text))
        return c

    def set_config(self, config):
        cfile = StringIO()
        config.write(cfile)
        cfile.seek(0)
        text = cfile.read()
        self.text = text


class GroupMixin(BaseShortNameUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'groups'

    @declared_attr
    def description(self):
        return Column(UnicodeText)

    @declared_attr
    def users(self):
        return relationship('User', secondary='group_user',
                            backref='groups')


class UserGroupMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'group_user'

    @declared_attr
    def group_id(self):
        return Column(UUIDType,
                      ForeignKey('groups.id',
                                 onupdate='CASCADE',
                                 ondelete='CASCADE'),
                      primary_key=True)

    @declared_attr
    def user_id(self):
        return Column(UUIDType,
                      ForeignKey('users.id',
                                 onupdate='CASCADE',
                                 ondelete='CASCADE'),
                      primary_key=True)


# User.config = relationship(UserConfig, uselist=False, lazy='subquery')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `hornstone` package."""

import io
import pytest

import hornstone
from hornstone.base import get_sha256sum, get_sha256sum_string
from hornstone.base import trailing_slash, remove_trailing_slash



@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


hello_hash = "2cf24dba5fb0a30e26e83b2ac5b9e29e1b161e5c1fa7425e73043362938b9824"


def test_hash_hello_file():
    hfile = io.BytesIO(b'hello')
    fhash = get_sha256sum(hfile)
    assert fhash == hello_hash


def test_hash_hello_string():
    hello = b'hello'
    fhash = get_sha256sum_string(hello)
    assert fhash == hello_hash

def test_trailing_slash():
    u = 'hello/'
    res = trailing_slash(u)
    assert res == u

def test_trailing_slash_needed():
    u = 'hello'
    res = trailing_slash(u)
    assert res == 'hello/'


def test_rm_slash1():
    u = "end/"
    res = remove_trailing_slash(u)
    assert res == "end"


def test_rm_slash2():
    u = "end"
    res = remove_trailing_slash(u)
    assert res == "end"

def test_rm_slash3():
    u = "end/////"
    res = remove_trailing_slash(u)
    assert res == "end"



